@@ -1,9 +1,9 @@
-use ruff_python_ast::{self as ast, Expr, Stmt};
+use ruff_python_ast::{Expr, Stmt};
 
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 
 /// ## What it does
@@ -27,23 +27,21 @@ use crate::checkers::ast::Checker;
 ///
 /// ## References
 /// - [Python documentation: The `assert` statement](https://docs.python.org/3/reference/simple_stmts.html#the-assert-statement)
-#[violation]
-pub struct AssertTuple;
+#[derive(ViolationMetadata)]
+pub(crate) struct AssertTuple;
 
 impl Violation for AssertTuple {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Assert test is a non-empty tuple, which is always `True`")
+        "Assert test is a non-empty tuple, which is always `True`".to_string()
     }
 }
 
 /// F631
-pub(crate) fn assert_tuple(checker: &mut Checker, stmt: &Stmt, test: &Expr) {
-    if let Expr::Tuple(ast::ExprTuple { elts, .. }) = &test {
-        if !elts.is_empty() {
-            checker
-                .diagnostics
-                .push(Diagnostic::new(AssertTuple, stmt.range()));
+pub(crate) fn assert_tuple(checker: &Checker, stmt: &Stmt, test: &Expr) {
+    if let Expr::Tuple(tuple) = &test {
+        if !tuple.is_empty() {
+            checker.report_diagnostic(AssertTuple, stmt.range());
         }
     }
 }
