@@ -5,7 +5,7 @@ use ruff_text_size::{Ranged, TextSize};
 
 use crate::checkers::ast::Checker;
 use crate::preview::is_safe_super_call_with_parameters_fix_enabled;
-use crate::{AlwaysFixableViolation, Edit, Fix};
+use crate::{Edit, Fix, FixAvailability, Violation};
 
 /// ## What it does
 /// Checks for `super` calls that pass redundant arguments.
@@ -57,14 +57,16 @@ use crate::{AlwaysFixableViolation, Edit, Fix};
 #[derive(ViolationMetadata)]
 pub(crate) struct SuperCallWithParameters;
 
-impl AlwaysFixableViolation for SuperCallWithParameters {
+impl Violation for SuperCallWithParameters {
+    const FIX_AVAILABILITY: FixAvailability = FixAvailability::Sometimes;
+
     #[derive_message_formats]
     fn message(&self) -> String {
         "Use `super()` instead of `super(__class__, self)`".to_string()
     }
 
-    fn fix_title(&self) -> String {
-        "Remove `super()` parameters".to_string()
+    fn fix_title(&self) -> Option<String> {
+        Some("Remove `super()` parameters".to_string())
     }
 }
 
@@ -165,22 +167,26 @@ pub(crate) fn super_call_with_parameters(checker: &Checker, call: &ast::ExprCall
         return;
     }
 
-    let applicability = if !checker.comment_ranges().intersects(call.arguments.range())
-        && is_safe_super_call_with_parameters_fix_enabled(checker.settings())
-    {
-        Applicability::Safe
-    } else {
-        Applicability::Unsafe
-    };
-
     let mut diagnostic = checker.report_diagnostic(SuperCallWithParameters, call.arguments.range());
-    diagnostic.set_fix(Fix::applicable_edit(
-        Edit::deletion(
-            call.arguments.start() + TextSize::new(1),
-            call.arguments.end() - TextSize::new(1),
-        ),
-        applicability,
-    ));
+
+    // Only provide a fix if there are no keyword arguments, since super() doesn't accept keyword arguments
+    if call.arguments.keywords.is_empty() {
+        let applicability = if !checker.comment_ranges().intersects(call.arguments.range())
+            && is_safe_super_call_with_parameters_fix_enabled(checker.settings())
+        {
+            Applicability::Safe
+        } else {
+            Applicability::Unsafe
+        };
+
+        diagnostic.set_fix(Fix::applicable_edit(
+            Edit::deletion(
+                call.arguments.start() + TextSize::new(1),
+                call.arguments.end() - TextSize::new(1),
+            ),
+            applicability,
+        ));
+    }
 }
 
 /// Returns `true` if a call is an argumented `super` invocation.
