@@ -50,20 +50,20 @@ mod tests {
 
         assert_snapshot!(test.goto_type_definition(), @r"
         info[goto-type-definition]: Type definition
-         --> main.py:2:19
+         --> main.py:2:7
           |
-        2 |             class Test: ...
-          |                   ^^^^
+        2 | class Test: ...
+          |       ^^^^
         3 |
-        4 |             ab = Test()
+        4 | ab = Test()
           |
         info: Source
-         --> main.py:4:13
+         --> main.py:4:1
           |
-        2 |             class Test: ...
+        2 | class Test: ...
         3 |
-        4 |             ab = Test()
-          |             ^^
+        4 | ab = Test()
+          | ^^
           |
         ");
     }
@@ -82,20 +82,20 @@ mod tests {
 
         assert_snapshot!(test.goto_type_definition(), @r"
         info[goto-type-definition]: Type definition
-         --> main.py:2:17
+         --> main.py:2:5
           |
-        2 |             def foo(a, b): ...
-          |                 ^^^
+        2 | def foo(a, b): ...
+          |     ^^^
         3 |
-        4 |             ab = foo
+        4 | ab = foo
           |
         info: Source
-         --> main.py:6:13
+         --> main.py:6:1
           |
-        4 |             ab = foo
+        4 | ab = foo
         5 |
-        6 |             ab
-          |             ^^
+        6 | ab
+          | ^^
           |
         ");
     }
@@ -120,39 +120,39 @@ mod tests {
 
         assert_snapshot!(test.goto_type_definition(), @r"
         info[goto-type-definition]: Type definition
-         --> main.py:3:17
+         --> main.py:3:5
           |
-        3 |             def foo(a, b): ...
-          |                 ^^^
+        3 | def foo(a, b): ...
+          |     ^^^
         4 |
-        5 |             def bar(a, b): ...
+        5 | def bar(a, b): ...
           |
         info: Source
-          --> main.py:12:13
+          --> main.py:12:1
            |
-        10 |                 a = bar
+        10 |     a = bar
         11 |
-        12 |             a
-           |             ^
+        12 | a
+           | ^
            |
 
         info[goto-type-definition]: Type definition
-         --> main.py:5:17
+         --> main.py:5:5
           |
-        3 |             def foo(a, b): ...
+        3 | def foo(a, b): ...
         4 |
-        5 |             def bar(a, b): ...
-          |                 ^^^
+        5 | def bar(a, b): ...
+          |     ^^^
         6 |
-        7 |             if random.choice():
+        7 | if random.choice():
           |
         info: Source
-          --> main.py:12:13
+          --> main.py:12:1
            |
-        10 |                 a = bar
+        10 |     a = bar
         11 |
-        12 |             a
-           |             ^
+        12 | a
+           | ^
            |
         ");
     }
@@ -177,12 +177,12 @@ mod tests {
           | ^^^^^^
           |
         info: Source
-         --> main.py:4:13
+         --> main.py:4:1
           |
-        2 |             import lib
+        2 | import lib
         3 |
-        4 |             lib
-          |             ^^^
+        4 | lib
+          | ^^^
           |
         ");
     }
@@ -199,22 +199,21 @@ mod tests {
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:892:7
+           --> stdlib/builtins.pyi:911:7
             |
-        890 |     def __getitem__(self, key: int, /) -> str | int | None: ...
-        891 |
-        892 | class str(Sequence[str]):
+        910 | @disjoint_base
+        911 | class str(Sequence[str]):
             |       ^^^
-        893 |     """str(object='') -> str
-        894 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        912 |     """str(object='') -> str
+        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
-         --> main.py:4:13
+         --> main.py:4:1
           |
-        2 |             a: str = "test"
+        2 | a: str = "test"
         3 |
-        4 |             a
-          |             ^
+        4 | a
+          | ^
           |
         "#);
     }
@@ -228,20 +227,19 @@ mod tests {
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:892:7
+           --> stdlib/builtins.pyi:911:7
             |
-        890 |     def __getitem__(self, key: int, /) -> str | int | None: ...
-        891 |
-        892 | class str(Sequence[str]):
+        910 | @disjoint_base
+        911 | class str(Sequence[str]):
             |       ^^^
-        893 |     """str(object='') -> str
-        894 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        912 |     """str(object='') -> str
+        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
-         --> main.py:2:22
+         --> main.py:2:10
           |
-        2 |             a: str = "test"
-          |                      ^^^^^^
+        2 | a: str = "test"
+          |          ^^^^^^
           |
         "#);
     }
@@ -256,16 +254,16 @@ mod tests {
 
         assert_snapshot!(test.goto_type_definition(), @r"
         info[goto-type-definition]: Type definition
-         --> main.py:2:24
+         --> main.py:2:12
           |
-        2 |             type Alias[T: int = bool] = list[T]
-          |                        ^
+        2 | type Alias[T: int = bool] = list[T]
+          |            ^
           |
         info: Source
-         --> main.py:2:46
+         --> main.py:2:34
           |
-        2 |             type Alias[T: int = bool] = list[T]
-          |                                              ^
+        2 | type Alias[T: int = bool] = list[T]
+          |                                  ^
           |
         ");
     }
@@ -312,22 +310,22 @@ mod tests {
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-         --> main.py:4:13
+         --> main.py:4:1
           |
-        2 |             from typing_extensions import TypeAliasType
+        2 | from typing_extensions import TypeAliasType
         3 |
-        4 |             Alias = TypeAliasType("Alias", tuple[int, int])
-          |             ^^^^^
+        4 | Alias = TypeAliasType("Alias", tuple[int, int])
+          | ^^^^^
         5 |
-        6 |             Alias
+        6 | Alias
           |
         info: Source
-         --> main.py:6:13
+         --> main.py:6:1
           |
-        4 |             Alias = TypeAliasType("Alias", tuple[int, int])
+        4 | Alias = TypeAliasType("Alias", tuple[int, int])
         5 |
-        6 |             Alias
-          |             ^^^^^
+        6 | Alias
+          | ^^^^^
           |
         "#);
     }
@@ -344,22 +342,21 @@ mod tests {
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:892:7
+           --> stdlib/builtins.pyi:911:7
             |
-        890 |     def __getitem__(self, key: int, /) -> str | int | None: ...
-        891 |
-        892 | class str(Sequence[str]):
+        910 | @disjoint_base
+        911 | class str(Sequence[str]):
             |       ^^^
-        893 |     """str(object='') -> str
-        894 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        912 |     """str(object='') -> str
+        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
-         --> main.py:4:18
+         --> main.py:4:6
           |
-        2 |             def test(a: str): ...
+        2 | def test(a: str): ...
         3 |
-        4 |             test(a= "123")
-          |                  ^
+        4 | test(a= "123")
+          |      ^
           |
         "#);
     }
@@ -379,22 +376,21 @@ mod tests {
         //   is an int. Navigating to `str` would match pyright's behavior.
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:337:7
+           --> stdlib/builtins.pyi:344:7
             |
-        335 | _LiteralInteger = _PositiveInteger | _NegativeInteger | Literal[0]  # noqa: Y026  # TODO: Use TypeAlias once mypy bugs are fixed
-        336 |
-        337 | class int:
+        343 | @disjoint_base
+        344 | class int:
             |       ^^^
-        338 |     """int([x]) -> integer
-        339 |     int(x, base=10) -> integer
+        345 |     """int([x]) -> integer
+        346 |     int(x, base=10) -> integer
             |
         info: Source
-         --> main.py:4:18
+         --> main.py:4:6
           |
-        2 |             def test(a: str): ...
+        2 | def test(a: str): ...
         3 |
-        4 |             test(a= 123)
-          |                  ^
+        4 | test(a= 123)
+          |      ^
           |
         "#);
     }
@@ -413,14 +409,13 @@ f(**kwargs<CURSOR>)
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-            --> stdlib/builtins.pyi:2890:7
+            --> stdlib/builtins.pyi:2917:7
              |
-        2888 |         """See PEP 585"""
-        2889 |
-        2890 | class dict(MutableMapping[_KT, _VT]):
+        2916 | @disjoint_base
+        2917 | class dict(MutableMapping[_KT, _VT]):
              |       ^^^^
-        2891 |     """dict() -> new empty dictionary
-        2892 |     dict(mapping) -> new dictionary initialized from a mapping object's
+        2918 |     """dict() -> new empty dictionary
+        2919 |     dict(mapping) -> new dictionary initialized from a mapping object's
              |
         info: Source
          --> main.py:6:5
@@ -444,21 +439,20 @@ f(**kwargs<CURSOR>)
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:892:7
+           --> stdlib/builtins.pyi:911:7
             |
-        890 |     def __getitem__(self, key: int, /) -> str | int | None: ...
-        891 |
-        892 | class str(Sequence[str]):
+        910 | @disjoint_base
+        911 | class str(Sequence[str]):
             |       ^^^
-        893 |     """str(object='') -> str
-        894 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        912 |     """str(object='') -> str
+        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
-         --> main.py:3:17
+         --> main.py:3:5
           |
-        2 |             def foo(a: str):
-        3 |                 a
-          |                 ^
+        2 | def foo(a: str):
+        3 |     a
+          |     ^
           |
         "#);
     }
@@ -478,19 +472,19 @@ f(**kwargs<CURSOR>)
 
         assert_snapshot!(test.goto_type_definition(), @r"
         info[goto-type-definition]: Type definition
-         --> main.py:2:19
+         --> main.py:2:7
           |
-        2 |             class X:
-          |                   ^
-        3 |                 def foo(a, b): ...
+        2 | class X:
+          |       ^
+        3 |     def foo(a, b): ...
           |
         info: Source
-         --> main.py:7:13
+         --> main.py:7:1
           |
-        5 |             x = X()
+        5 | x = X()
         6 |
-        7 |             x.foo()
-          |             ^
+        7 | x.foo()
+          | ^
           |
         ");
     }
@@ -507,20 +501,20 @@ f(**kwargs<CURSOR>)
 
         assert_snapshot!(test.goto_type_definition(), @r"
         info[goto-type-definition]: Type definition
-         --> main.py:2:17
+         --> main.py:2:5
           |
-        2 |             def foo(a, b): ...
-          |                 ^^^
+        2 | def foo(a, b): ...
+          |     ^^^
         3 |
-        4 |             foo()
+        4 | foo()
           |
         info: Source
-         --> main.py:4:13
+         --> main.py:4:1
           |
-        2 |             def foo(a, b): ...
+        2 | def foo(a, b): ...
         3 |
-        4 |             foo()
-          |             ^^^
+        4 | foo()
+          | ^^^
           |
         ");
     }
@@ -537,22 +531,21 @@ f(**kwargs<CURSOR>)
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:892:7
+           --> stdlib/builtins.pyi:911:7
             |
-        890 |     def __getitem__(self, key: int, /) -> str | int | None: ...
-        891 |
-        892 | class str(Sequence[str]):
+        910 | @disjoint_base
+        911 | class str(Sequence[str]):
             |       ^^^
-        893 |     """str(object='') -> str
-        894 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        912 |     """str(object='') -> str
+        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
-         --> main.py:4:27
+         --> main.py:4:15
           |
-        2 |             def foo(a: str | None, b):
-        3 |                 if a is not None:
-        4 |                     print(a)
-          |                           ^
+        2 | def foo(a: str | None, b):
+        3 |     if a is not None:
+        4 |         print(a)
+          |               ^
           |
         "#);
     }
@@ -568,38 +561,37 @@ f(**kwargs<CURSOR>)
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/types.pyi:922:11
+           --> stdlib/types.pyi:941:11
             |
-        920 | if sys.version_info >= (3, 10):
-        921 |     @final
-        922 |     class NoneType:
+        939 | if sys.version_info >= (3, 10):
+        940 |     @final
+        941 |     class NoneType:
             |           ^^^^^^^^
-        923 |         """The type of the None singleton."""
+        942 |         """The type of the None singleton."""
             |
         info: Source
-         --> main.py:3:17
+         --> main.py:3:5
           |
-        2 |             def foo(a: str | None, b):
-        3 |                 a
-          |                 ^
+        2 | def foo(a: str | None, b):
+        3 |     a
+          |     ^
           |
 
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:892:7
+           --> stdlib/builtins.pyi:911:7
             |
-        890 |     def __getitem__(self, key: int, /) -> str | int | None: ...
-        891 |
-        892 | class str(Sequence[str]):
+        910 | @disjoint_base
+        911 | class str(Sequence[str]):
             |       ^^^
-        893 |     """str(object='') -> str
-        894 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        912 |     """str(object='') -> str
+        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
-         --> main.py:3:17
+         --> main.py:3:5
           |
-        2 |             def foo(a: str | None, b):
-        3 |                 a
-          |                 ^
+        2 | def foo(a: str | None, b):
+        3 |     a
+          |     ^
           |
         "#);
     }
