@@ -427,7 +427,7 @@ impl Session {
     /// Returns a mutable iterator over all project databases that have been initialized to this point.
     ///
     /// This iterator will only yield the default project database if it has been used.
-    fn projects_mut(&mut self) -> impl Iterator<Item = &'_ mut ProjectDatabase> + '_ {
+    pub(crate) fn projects_mut(&mut self) -> impl Iterator<Item = &'_ mut ProjectDatabase> + '_ {
         self.project_states_mut().map(|project| &mut project.db)
     }
 
@@ -826,6 +826,7 @@ impl Session {
             .map_err(DocumentQueryError::InvalidUrl);
         DocumentSnapshot {
             resolved_client_capabilities: self.resolved_client_capabilities,
+            global_settings: self.global_settings.clone(),
             workspace_settings: key
                 .as_ref()
                 .ok()
@@ -1000,6 +1001,7 @@ impl Drop for MutIndexGuard<'_> {
 #[derive(Debug)]
 pub(crate) struct DocumentSnapshot {
     resolved_client_capabilities: ResolvedClientCapabilities,
+    global_settings: Arc<GlobalSettings>,
     workspace_settings: Arc<WorkspaceSettings>,
     position_encoding: PositionEncoding,
     document_query_result: Result<DocumentQuery, DocumentQueryError>,
@@ -1016,6 +1018,11 @@ impl DocumentSnapshot {
         self.position_encoding
     }
 
+    /// Returns the client settings for all workspaces.
+    pub(crate) fn global_settings(&self) -> &GlobalSettings {
+        &self.global_settings
+    }
+
     /// Returns the client settings for the workspace that this document belongs to.
     pub(crate) fn workspace_settings(&self) -> &WorkspaceSettings {
         &self.workspace_settings
